﻿// Created by Brock, Boris, 2015-06-29
// Copyright (c) Boris Brock. All rights reserved.

#region Usings

using System;
using System.ComponentModel;
using System.Globalization;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using DeskControl.Properties;
using DeskControl.Sources.Serial;

#endregion

namespace DeskControl.Sources.UI
{
  /// <summary>
  ///   Interaction logic for MainWindow.xaml
  /// </summary>
  public partial class MainWindow
  {
    private readonly ArduinoConnector mArduino;


    /// <summary>
    ///   Initializes a new instance of the <see cref="MainWindow" /> class.
    /// </summary>
    public MainWindow()
    {
      InitializeComponent();

      // Get port from command line
      var portName = GetPortNameFromCommandLine();
      if (string.IsNullOrEmpty(portName))
      {
        MessageBox.Show("No COM port name was given via command line. Shutting down...", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
        Close();
      }
      else
      {
        mArduino = new ArduinoConnector();
        mArduino.ConnectionFailed += OnArduinoConnectionFailed;
        mArduino.ConnectionSucceeded += OnArduinoConnectionSucceeded;
        mArduino.TableHeightChanged += OnTableHeightChanged;
        mArduino.ConnectAsync(portName);
      }

      // Restore the window position
      Top = Settings.Default.WindowPosY;
      Left = Settings.Default.WindowPosX;
    }

    /// <summary>
    ///   Handles the MouseDown event of the BtnHeight control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.Windows.Input.MouseButtonEventArgs" /> instance containing the event data.</param>
    private void BtnHeight_MouseDown(object sender, MouseButtonEventArgs e)
    {
      if (mArduino.IsConnected)
      {
        var lbl = sender as Label;
        if (lbl == null) return;

        var height = Int32.Parse(lbl.Content.ToString());
        mArduino.MoveTableToHeight(height);
      }
    }

    /// <summary>
    ///   Gets the port name from command line.
    /// </summary>
    /// <returns></returns>
    private string GetPortNameFromCommandLine()
    {
      string[] args = Environment.GetCommandLineArgs();
      if (args.Length < 2) return string.Empty;
      return args[1];
    }

    /// <summary>
    ///   Handles the MouseDown event of the LblHeight control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="MouseButtonEventArgs" /> instance containing the event data.</param>
    private void LblHeight_MouseDown(object sender, MouseButtonEventArgs e)
    {
      mArduino.StopTable();
    }

    /// <summary>
    ///   Handles the Closing event of the MetroWindow control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs" /> instance containing the event data.</param>
    private void MetroWindow_Closing(object sender, CancelEventArgs e)
    {
      mArduino.StopTable();
      mArduino.Disconnect();

      // Save the window position
      Settings.Default.WindowPosY = (int) Top;
      Settings.Default.WindowPosX = (int) Left;
      Settings.Default.Save();
    }

    /// <summary>
    ///   Called when [arduino connection failed].
    /// </summary>
    private void OnArduinoConnectionFailed()
    {
      Dispatcher.Invoke(() =>
      {
        LblHeight.Foreground = Brushes.Crimson;
        LblHeight.Content = "?";
      });
    }

    /// <summary>
    ///   Called when [arduino connection succeeded].
    /// </summary>
    private void OnArduinoConnectionSucceeded()
    {
      if (mArduino.CurrentTableHeight > 0) UpdateHeight(mArduino.CurrentTableHeight);
      else mArduino.MoveUpUntilHeightIsReceived();
    }

    /// <summary>
    ///   Handles the on height changed event.
    /// </summary>
    /// <param name="height">The height.</param>
    /// <exception cref="System.NotImplementedException"></exception>
    private void OnTableHeightChanged(Int32 height)
    {
      UpdateHeight(height);
    }

    /// <summary>
    ///   Updates the height.
    /// </summary>
    /// <param name="height">The height.</param>
    private void UpdateHeight(Int32 height)
    {
      Dispatcher.Invoke(() => LblHeight.Content = height.ToString(CultureInfo.InvariantCulture));
    }
  }
}